// OptionProject.cpp : implementation file
//

#include "stdafx.h"
#include "base.h"
#include "OptionProject.h"
#include "utils.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// COptionProject dialog


COptionProject::COptionProject(CWnd* pParent /*=NULL*/)
	: CDialog(COptionProject::IDD, pParent)
{
	//{{AFX_DATA_INIT(COptionProject)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void COptionProject::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COptionProject)
	DDX_Control(pDX, IDC_PROJECT, m_Tree);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(COptionProject, CDialog)
	//{{AFX_MSG_MAP(COptionProject)
	ON_BN_CLICKED(IDC_BUTTONREFRESH, OnButtonRefresh)
	ON_NOTIFY(NM_DBLCLK, IDC_PROJECT, OnDoubleClick)
	ON_BN_CLICKED(IDC_BUTTONDEFEDIT, OnDefaultEdit)
	ON_BN_CLICKED(IDC_BUTTONEDIT, OnButtonNotepad)
	ON_NOTIFY(TVN_ITEMEXPANDED, IDC_PROJECT, OnItemExpanded)
	ON_BN_CLICKED(IDC_BUTTONDELETE, OnButtonDelete)
	ON_NOTIFY(TVN_SELCHANGED, IDC_PROJECT, OnSelectionChanged)
	ON_WM_SHOWWINDOW()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COptionProject message handlers


BOOL COptionProject::OnInitDialog() 
{
	CDialog::OnInitDialog();

	themeDir = "";

	CImageList* pImageList = new CImageList();
	pImageList->Create(16, 16, ILC_COLOR24, 6, 2);
	pImageList->SetBkColor(RGB(255, 255, 255));

	idFolder	= pImageList->Add(AfxGetApp()->LoadIcon(IDI_FOLDER));
	idSelFolder	= pImageList->Add(AfxGetApp()->LoadIcon(IDI_SELFOLDER));
	idSpecFolder= pImageList->Add(AfxGetApp()->LoadIcon(IDI_SPECFOLDER));
	idRc		= pImageList->Add(AfxGetApp()->LoadIcon(IDI_RC));
	idTxt		= pImageList->Add(AfxGetApp()->LoadIcon(IDI_TXT));
	idIni		= pImageList->Add(AfxGetApp()->LoadIcon(IDI_INI));

	m_Tree.SetImageList(pImageList, TVSIL_NORMAL);

	LoadProjectTree();
	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// 
// Get the path of the selected item
// Return value is 'false' when selected item is first-level special dir,
// i.e. PersonalDir or ThemeDir
//
bool COptionProject::GetSelectedFilePath(CString &file)
{
	HTREEITEM		hItem;
	CString			szFile;

	hItem = m_Tree.GetSelectedItem();	// get a handle to the selected item
	if (m_Tree.GetParentItem(hItem) == NULL)
	{
		if (m_Tree.GetItemText(hItem) != "step.rc")
			return false;				// cannot edit first level folders :)
		file = utils.litestepDir + "step.rc";
	}
	else
	{
		szFile = m_Tree.GetItemText(hItem);	// Get the text of the chosen option.
		hItem = m_Tree.GetParentItem(hItem);
		while (m_Tree.GetParentItem(hItem) != NULL)
		{
			szFile = m_Tree.GetItemText(hItem) + "\\" + szFile;
			hItem = m_Tree.GetParentItem(hItem);
		}
		if (m_Tree.GetItemText(hItem) == "Personal Files")
			szFile = utils.personalDir + szFile;
		else
			szFile = themeDir + szFile;
		file = szFile;
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
//
// Generate .config-files tree
//
void COptionProject::LoadProjectTree()
{
	m_Tree.DeleteAllItems();
	HTREEITEM hItem;
	hItem = m_Tree.InsertItem("Personal Files", idSpecFolder, idSpecFolder);
	LoadFolder(hItem, utils.personalDir);
	hItem = m_Tree.InsertItem("Theme Project", idFolder, idFolder);
	CString csThemeDir = utils.ReadOptionFromRC(utils.themesDir + "themeselect.rc","ThemeDir");
	if (csThemeDir == "")
		m_Tree.InsertItem("ThemeDir variable not available...",hItem);
	else
	{
		themeDir = utils.ExpandString(csThemeDir);
		CString csThemeName = utils.ReadOptionFromRC(themeDir + "theme.rc","ThemeName");
		if (csThemeName != "")
			m_Tree.SetItemText(hItem,"Theme Project (" +  csThemeName + ")");
		LoadFolder(hItem, themeDir);
		m_Tree.Expand(hItem,TVE_EXPAND);
		m_Tree.SelectItem(hItem);
	}
	hItemStepRc = m_Tree.InsertItem("step.rc", idRc, idRc);
}

//////////////////////////////////////////////////////////////////////////////////
//
// List specified files and add them to the tree
//
void COptionProject::EnumFiles(HTREEITEM hParent, CString csPattern, int img_id)
{
	HANDLE hFind;
	WIN32_FIND_DATA dataFind;
	BOOL bMoreFiles = TRUE;
	HTREEITEM hItem;

	// find the first file
	hFind = FindFirstFile(csPattern, &dataFind);
	bMoreFiles = TRUE;
	while (hFind != INVALID_HANDLE_VALUE && bMoreFiles == TRUE)
	{
		if (!(dataFind.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
		{
			hItem = m_Tree.InsertItem(dataFind.cFileName, img_id, img_id, hParent);
			m_Tree.SetItemData(hItem,dataFind.dwFileAttributes);
		}
		bMoreFiles = FindNextFile(hFind, &dataFind);
	}
	FindClose(hFind);
}

///////////////////////////////////////////////////////////////////////////////////
//
// traverse the directory and list all .config-files
//
void COptionProject::LoadFolder(HTREEITEM hParent, CString folder)
{
	HANDLE hFind;
	WIN32_FIND_DATA dataFind;
	BOOL bMoreFiles = TRUE;

	// directories

	hFind = FindFirstFile(folder + "*.*", &dataFind);
	while (hFind != INVALID_HANDLE_VALUE && bMoreFiles == TRUE)
	{
		if (dataFind.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
		{
			if (strcmp(dataFind.cFileName, ".") && strcmp(dataFind.cFileName, ".."))
			{
				HTREEITEM hItem = m_Tree.InsertItem(dataFind.cFileName, idFolder, idFolder, hParent);
				m_Tree.SetItemData(hItem,dataFind.dwFileAttributes);
				LoadFolder(hItem, folder  + dataFind.cFileName + (CString)"\\");
			}
		}
		bMoreFiles = FindNextFile(hFind, &dataFind);
	}
	FindClose(hFind);

	EnumFiles(hParent, folder + "*.rc",  idRc);
	EnumFiles(hParent, folder + "*.box", idRc);
	EnumFiles(hParent, folder + "*.ini", idIni);
	EnumFiles(hParent, folder + "*.txt", idTxt);
}

void COptionProject::OnButtonRefresh() 
{
	LoadProjectTree();	
}

void COptionProject::OnDefaultEdit() 
{
	CString csFile;
	if (GetSelectedFilePath(csFile))
	{
		if (! (m_Tree.GetItemData(m_Tree.GetSelectedItem()) & FILE_ATTRIBUTE_DIRECTORY))
			ShellExecute(NULL,NULL,"\"" + csFile + "\"",NULL,"",SW_SHOWNORMAL);
		else
			AfxMessageBox("Cannot open file:\n" + csFile);
	}
}


void COptionProject::OnButtonNotepad() 
{
	CString csFile;
	if (GetSelectedFilePath(csFile))
	{
		if (! (m_Tree.GetItemData(m_Tree.GetSelectedItem()) & FILE_ATTRIBUTE_DIRECTORY))
			ShellExecute(NULL,NULL,"notepad.exe", "\"" + csFile + "\"","",SW_SHOWNORMAL);
		else
			AfxMessageBox("Cannot open file:\n" + csFile);
	}	
}

void COptionProject::OnButtonDelete()
{
	CString csFile;
	if (GetSelectedFilePath(csFile))
	{
		SHFILEOPSTRUCT shfop;
		shfop.hwnd = m_hWnd;
		shfop.wFunc = FO_DELETE;
		shfop.pFrom = csFile + (char)0;
		shfop.pTo = NULL;
		shfop.fFlags = 0;
		SHFileOperation(&shfop);
		m_Tree.DeleteItem(m_Tree.GetSelectedItem());
	}
}

void COptionProject::OnSelectionChanged(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	CString csFile;
	if (GetSelectedFilePath(csFile))
	{
		GetDlgItem(IDC_BUTTONDELETE)->EnableWindow(
			m_Tree.GetSelectedItem() == hItemStepRc ? FALSE : TRUE
			);
		if (m_Tree.GetItemData(m_Tree.GetSelectedItem()) & FILE_ATTRIBUTE_DIRECTORY)
		{
			GetDlgItem(IDC_BUTTONEDIT)->EnableWindow(FALSE);
			GetDlgItem(IDC_BUTTONDEFEDIT)->EnableWindow(FALSE);
		}
		else
		{
			GetDlgItem(IDC_BUTTONEDIT)->EnableWindow();
			GetDlgItem(IDC_BUTTONDEFEDIT)->EnableWindow();
		}
	} else
	{
		GetDlgItem(IDC_BUTTONDELETE)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTONEDIT)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTONDEFEDIT)->EnableWindow(FALSE);
	}
	*pResult = 0;
}

void COptionProject::OnDoubleClick(NMHDR* pNMHDR, LRESULT* pResult) 
{	
	*pResult = 0;
	if (! (m_Tree.GetItemData(m_Tree.GetSelectedItem()) & FILE_ATTRIBUTE_DIRECTORY) )
	{
		OnDefaultEdit();
	}
}

void COptionProject::OnItemExpanded(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	HTREEITEM hItem = pNMTreeView->itemNew.hItem;
	if (!(m_Tree.GetParentItem(hItem) == NULL && m_Tree.GetItemText(hItem) == "Personal Files"))
	{
		switch (pNMTreeView->action)
		{
		case TVE_EXPAND:
		case TVE_EXPANDPARTIAL:
			m_Tree.SetItemImage(hItem, idSelFolder, idSelFolder);
			break;
		case TVE_COLLAPSE:
		case TVE_COLLAPSERESET:
			m_Tree.SetItemImage(hItem, idFolder, idFolder);
			break;
		}
	}
	*pResult = 0;
}


void COptionProject::OnShowWindow(BOOL bShow, UINT nStatus) 
{
	CDialog::OnShowWindow(bShow, nStatus);	
	if (bShow)	LoadProjectTree();
}
